/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.IllegalPositionException;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.Player;
import java.util.ArrayList;
import java.util.List;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.adapters.XmlAdapter;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public final class Game extends Object {
    @XmlElement
    private GameId id;
    @XmlElement
    @XmlJavaTypeAdapter(BoardAdapter.class)
    private Board board;
    @XmlElement
    @XmlJavaTypeAdapter(MoveAdapter.class)
    private List<Move> moves;

    Game() {
    }

    public Game(String first, String second) {
        this.id = new GameId(first, second);
        this.board = Board.empty();
    }

    public Game(GameId id) {
        this.id = id;
        this.board = Board.empty();
    }

    public GameId getId() {
        return id;
    }
    
    public Board getBoard() {
        return board;
    }

    public void apply(String player, Move m) throws IllegalPositionException {
        Player p = null;
        if (id.getWhite().equals(player)) {
            p = getBoard().getPlayers().get(0);
        } else {
            if (id.getBlack().equals(player)) {
                p = getBoard().getPlayers().get(1);
            }
        }
        if (p != getBoard().getCurrentPlayer()) {
            throw new IllegalArgumentException("Wrong player: " + player);
        }

        board = getBoard().apply(m);
        getMoves().add(m);
    }

    public List<Move> getMoves() {
        if (moves == null) {
            moves = new ArrayList<Move>();
        }
        return moves;
    }

    private static final class MoveAdapter extends XmlAdapter<String[],List<Move>> {
        @Override
        public List<Move> unmarshal(String[] arr) throws Exception {
            List<Move> res = new ArrayList<Move>();
            if (arr != null) {
                for (String v : arr) {
                    res.add(Move.valueOf(v));
                }
            }
            return res;
        }

        @Override
        public String[] marshal(List<Move> arr) throws Exception {
            List<String> res = new ArrayList<String>();
            if (arr != null) {
                for (Move m : arr) {
                    res.add(m.toString());
                }
            }
            return res.toArray(new String[0]);
        }
    } // end of MoveAdapter

    private static final class BoardAdapter extends XmlAdapter<String,Board> {

        @Override
        public Board unmarshal(String v) throws Exception {
            return v == null ? null : Board.valueOf(v);
        }

        @Override
        public String marshal(Board v) throws Exception {
            return v == null ? null : v.toString();
        }

    }
}
